/*****************************************************************************
 *
 * cgi_bin_summary.c - Defintion of the binary summary tables.
 *
 * Copyright (c) 2004 Ben Clewett (bclewett@perfparse.org)
 * Last Modified:   2004-05-28
 *
 * License:
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *****************************************************************************/

#include <libpp_common.h>
#include <config.h>
#include <gettext.h>
#include "common.h"
#include "cgi_bin_summary_policy.h"

/* Module CGI */
struct _cgi {
	int iFrequency;
	int iDefaultDelete;
	char *sNotes;
	char sFrequencyUnit[2];
} cgi;

/* Internals */
char *niceFrequency(int iHours);
void saveBinSummary(int iFrequency, char *sNotes, int iDefaultDelete);
void getCGI();

void displayDefineBinSummary()
{
	int bHit = FALSE;
	char sURL[1024];
	int iEdit;
	int iMetricCount;
	static const int iApproxRecSize = 159;
	int iTotalTableSpace = 0;
	int iCols;

	#ifdef USE_DB_POSTGRESQL
	int numberOfRows;
	#endif

	iCols = (!strcmp(config_get_value_as_string(CONFIG_ITEM_ID_DEFAULT_USER_PERMISSIONS_SUMMARY), USER_PERMISSIONS_RW)) ? 6 : 5;

	iMetricCount = query_int("SELECT COUNT(*) FROM perfdata_service_metric");

	showHeader(_("Definition of Binary Summary Data"), "");

	printf("<p align=center><table width=\"600\" border=1 cellpadding=5 cellspacing=0 bgcolor=\"#CCCCCC\"><tr><td>%s\n", FONT);
	printf("%s<br>\n", _("Binary Summary Data is used to store a summary of the Binary Data into fixed period samples."));
	printf("<ul>\n");
	printf("<li>%s</li>\n", _("This allows the storage of a very large history of Binary Data using small table space."));
	printf("<li>%s</li>\n", _("The data is stored as Average, Max, Min and Standard Deviation in each sample period."));
	printf("<li>%s</li>\n", _("Good values for sample periods are: 1 Hour, 4 Hours, 1 Day and 1 Week."));
	printf("<li>%s</li>\n", _("A maximum of 10000 sample periods is advised for a single graph."));
	printf("<li>%s</li>\n", _("Some of these options may be hard to change.  Please select values carefully."));
	printf("</ul>\n");
	printf(_("Unfortunatelly the current version of PerfParse does not use this data.  Data will be collected only.  Module to analyse data wil be completed in a future version."));
	printf("</font></td></tr></table></p>\n");

	/* Save data */

	if (icgi("submit", 0) && strcmp(scgi("enter"), _("Cancel"))) {

		/* Get the data */
		getCGI();

		/* Save the data */
		if (cgi.iFrequency) saveBinSummary(cgi.iFrequency, cgi.sNotes, cgi.iDefaultDelete);

	}
	
	/* Show the current values */

	g_string_printf(s_SQL, "SELECT frequency, notes, default_delete FROM perfdata_bin_summary ");
    query(s_SQL->str);

    #ifdef USE_DB_MYSQL
    while ((result_row = mysql_fetch_row(query_result))) {
    #elif defined USE_DB_POSTGRESQL
    numberOfRows = rows();
    for (currentRow = 0; currentRow<numberOfRows; currentRow++){
    #endif

		int iMaxSize, iFre, iMax;
		int iSize;

		if (!bHit) {
			bHit = TRUE;
			printf("<p align=center>\n");
			printf("<table width=\"100%%\" border=1 cellpadding=2 bgcolor=\"#FFFFCC\" cellspacing=0>\n");
			printf("<tr>\n");
			printf("  <td align=center>%s<b>%s</b></font></td>\n", FONT, _("Frequency"));
			printf("  <td align=center>%s<b>%s</b></font></td>\n", FONT, _("Default Samples"));
			printf("  <td align=center>%s<b>%s</b></font></td>\n", FONT, _("Max Data Age"));
			printf("  <td align=center>%s<b>%s*</b></font></td>\n", FONT, _("Max Table Space"));
			printf("  <td align=center>%s<b>%s</b></font></td>\n", FONT, _("Notes"));
			if (!strcmp(config_get_value_as_string(CONFIG_ITEM_ID_DEFAULT_USER_PERMISSIONS_SUMMARY), USER_PERMISSIONS_RW)) {
				printf("  <td align=center width=\"1%%\">%s<b>%s</b></font></td>\n", FONT, _("Edit"));
			}
			printf("</tr>\n");
		}

		printf("<tr bgcolor=\"#FFFFFF\">\n");

		iFre = iData(0);
		iMax = iData(2);
		iSize = (iMax * iMetricCount * iApproxRecSize) / (1024 * 1024);

		printf("  <td>%s\n", FONT);
		printf("%s\n", niceFrequency(iFre));
		printf("  </td>\n");

		printf("  <td align=\"right\">%s\n", FONT);
		printf("%d\n", iMax);
		printf("  </td>\n");

		iMaxSize = iFre * iMax;
		printf("  <td>%s\n", FONT);
		printf("%s\n", niceFrequency(iMaxSize));
		printf("  </td>\n");

		printf("  <td align=\"right\">%s\n", FONT);
		printf("%d MB\n", iSize);
		printf("  </td>\n");
		iTotalTableSpace += iSize;

		printf("  <td>%s\n", FONT);
		printf("%s&nbsp;\n", sData(1));
		printf("  </td>\n");

		if (!strcmp(config_get_value_as_string(CONFIG_ITEM_ID_DEFAULT_USER_PERMISSIONS_SUMMARY), USER_PERMISSIONS_RW)) {
			sprintf(sURL, "%s?define_bin_summary=1&edit=1&frequency=%d", sProgName, iData(0));
			printf("  <td>%s\n", FONT);
			printf("<a href=\"%s\">%s</a>\n", sURL, _("Edit"));
			printf("  </td>\n");
		}

		printf("</tr>\n");

	}

	if (bHit) {
		printf("<tr bgcolor=\"#FFFFFF\">\n");
		printf("  <td colspan=3 align=right>%s%s</td>\n", FONT, _("Total Table Space:"));
		printf("  <td align=right>%s%d MB</td>\n", FONT, iTotalTableSpace);
		printf("  <td colspan=%d>&nbsp;</td>\n", iCols - 4);
		printf("</tr>\n");
		printf("<tr><td colspan=%d>%s* %s %d %s %d %s</font></td></tr>\n", iCols, FONT, 
			_("Based on current"), iMetricCount, 
			_("Metrics and approximate record size of"), iApproxRecSize, 
			_("bytes."));
		printf("</table>\n</p>\n");
	}

	fflush(stdout);
	


	/* Add / Edit New */

	if (!strcmp(config_get_value_as_string(CONFIG_ITEM_ID_DEFAULT_USER_PERMISSIONS_SUMMARY), USER_PERMISSIONS_RW)) {

		iEdit = icgi("edit", 0);

		if (iEdit) {

			cgi.iFrequency = icgi("frequency", 0);
			g_string_printf(s_SQL, "SELECT notes, default_delete FROM perfdata_bin_summary WHERE frequency = %d", 
				cgi.iFrequency);
			query(s_SQL->str);
			
			#ifdef USE_DB_MYSQL
			result_row = mysql_fetch_row(query_result);
			#elif defined USE_DB_POSTGRESQL
			currentRow = 0;
			#endif

			cgi.sNotes = strdup(sData(0));
			cgi.iDefaultDelete = iData(1);
	
			// Adjust frequency to display form:
			if (cgi.iFrequency % (24*7) == 0 && cgi.iFrequency / (24*7) > 0) {
				cgi.iFrequency /= 24*7;
				strcpy(cgi.sFrequencyUnit, "w");
			}
			else if (cgi.iFrequency % (24) == 0 && cgi.iFrequency / (24) > 0) {
				cgi.iFrequency /= 24;
				strcpy(cgi.sFrequencyUnit, "d");
			}
			else {
				strcpy(cgi.sFrequencyUnit, "h");
			}
	
		} else {
			cgi.iFrequency = 0;
			cgi.sNotes = "";
			cgi.iDefaultDelete = 1000;
			cgi.sFrequencyUnit[0] = 'h';
		}


		printf("<form method=\"get\">\n");
		printf("<input type=\"hidden\" name=\"define_bin_summary\" value=\"1\">\n");
		printf("<input type=\"hidden\" name=\"submit\" value=\"1\">\n");
	
		printf("<p align=center>\n");
		printf("<table width=600 border=1 cellpadding=2 bgcolor=\"#FFFFCC\" cellspacing=0>\n");
		printf("<tr>\n");
		printf("  <td align=center>%s<b>%s</b></font></td>\n", FONT, 
			(iEdit) ? _("Edit Summary Sample Details") : _("Add New Summary Sample"));
		printf("</tr>\n");
		printf("<tr bgcolor=\"#FFFFFF\"><td>\n");
	
		printf("  <table width=\"100%%\" border=0 cellpadding=2 cellspacing=0>\n");
	
		printf("  <tr>\n");
		printf("    <td align=right>%s<b>%s:</td>\n", FONT, _("Frequency"));
		printf("    <td>\n");
		printf("      <input type=text name=\"frequency\" value=\"%d\" size=5 maxlength=5>", cgi.iFrequency);
		printf("      <select name=\"frequency_unit\">\n");
		printf("      <option value=\"h\"%s>Hours", (cgi.sFrequencyUnit[0] == 'h') ? " selected" : "");
		printf("      <option value=\"d\"%s>Days", (cgi.sFrequencyUnit[0] == 'd') ? " selected" : "");
		printf("      <option value=\"w\"%s>Weeks", (cgi.sFrequencyUnit[0] == 'w') ? " selected" : "");
		printf("      </select>\n");
		printf("    </td>\n");
		printf("  </tr>\n");
	
		printf("  <tr>\n");
		printf("    <td align=right>%s<b>%s:</td>\n", FONT, _("Default Max Samples"));
		printf("    <td><input name=\"default_delete\" size=6 maxlength=6 value=\"%d\"></td>\n", cgi.iDefaultDelete);
		printf("  </tr>\n");
	
		printf("  <tr>\n");
		printf("    <td align=right>%s<b>%s:</td>\n", FONT, _("Notes"));
		printf("    <td><textarea name=\"notes\" cols=30 rows=3 wrap=\"virtual\">%s</textarea></td>\n", cgi.sNotes);
		printf("  </tr>\n");
	
		printf("  </table>\n");

		printf("</td></tr><tr bgcolor=\"#FFFFCC\"><td align=center>\n");

		printf("  <input type=\"submit\" name=\"enter\" value=\"%s\">\n", _("Save"));
		if (iEdit) {
			printf("  <input type=\"submit\" name=\"enter\" value=\"%s\">\n", _("Cancel"));
		}
	
		printf("</td></tr></table></p></form>\n");
	}
			
	
	
}



void saveBinSummary(int iFrequency, char *sNotes, int iDefaultDelete)
{
	int iAmend;

	g_string_printf(s_SQL, "SELECT COUNT(*) FROM perfdata_bin_summary WHERE frequency = %d", iFrequency);
	iAmend = (query_int(s_SQL->str) == 0) ? FALSE : TRUE;

	if (iAmend) {

		g_string_printf(s_SQL, "UPDATE perfdata_bin_summary SET default_delete = %d, notes = '%s'",
			iDefaultDelete, sql_escape(sNotes, PERCENT_ESCAPE_CHAR));
		g_string_append_printf(s_SQL, " WHERE frequency = %d", iFrequency);

	} else {

		g_string_printf(s_SQL, "INSERT INTO perfdata_bin_summary (frequency, notes, default_delete) VALUES (%d, '%s', %d)",
			iFrequency, sql_escape(sNotes, PERCENT_ESCAPE_CHAR), iDefaultDelete);
	}

	query_no_return(s_SQL->str);


}

char *niceFrequency(int iHours)
{

	int iDays;
	int iWeeks;
	int iYears;
	static char sOut[80];

	sOut[0] = 0;

	iYears = iHours / (24*365);
	iHours = iHours % (24*365);

	iWeeks = iHours / (24*7);
	iHours = iHours % (24*7);

	iDays = iHours / 24;
	iHours = iHours % 24;

	if (iYears) {
		sprintf(sOut, "%s%s%d&nbsp;%s", sOut, (sOut[0]) ? ", " : "", iYears, (iYears == 1) ? _("Year") : _("Years"));
	}
	if (iWeeks) {
		sprintf(sOut, "%s%s%d&nbsp;%s", sOut, (sOut[0]) ? ", " : "", iWeeks, (iWeeks == 1) ? _("Week") : _("Weeks"));
	}
	if (iDays) {
		sprintf(sOut, "%s%s%d&nbsp;%s", sOut, (sOut[0]) ? ", " : "", iDays, (iDays == 1) ? _("Day") : _("Days"));
	}
	if (iHours) {
		sprintf(sOut, "%s%s%d&nbsp;%s", sOut, (sOut[0]) ? ", " : "", iHours, (iHours == 1) ? _("Hour") : _("Hours"));
	}

	return sOut;

}


void getCGI() 
{
	cgi.iFrequency = icgi("frequency", 0);

	strncpy(cgi.sFrequencyUnit, scgi("frequency_unit"), 1);
	cgi.sFrequencyUnit[1] = 0;

	switch (cgi.sFrequencyUnit[0]) {
		case 'd':	
			cgi.iFrequency *= 24;		
			break;
		case 'w':	
			cgi.iFrequency *= 24*7;		
			break;
	}

	cgi.iDefaultDelete = icgi("default_delete", 1000);

	cgi.sNotes = strdup(scgi("notes"));

}




