/*****************************************************************************
 *
 * perfparse-common.c - Parse Nagios data and store in relational database.
 *
 * Copyright (c) 2004 Yves Mettier (ymettier@libertysurf.fr)
 * Last Modified:   2004-09-01
 *
 * License:
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *****************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <errno.h>
#include <string.h>
#include <unistd.h>
#include <sys/types.h>
#include <signal.h>
#include <time.h>
#include <glib.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <config.h>
#include <sys/utsname.h>

#include "storage.h"

#include <libpp_common.h>
#include <gettext.h>

void registry_update(void) {
	time_t tm;
	struct utsname myuname;

	tm = time(NULL);
	storage_set_registry(config_get_value_as_string(CONFIG_ITEM_ID_DUMMY_HOSTNAME),"pp/perfparse/version",VERSION,tm);
	storage_set_registry(config_get_value_as_string(CONFIG_ITEM_ID_DUMMY_HOSTNAME),"pp/perfparse/compilation date",__DATE__ " " __TIME__,tm);
	if (-1 != uname (&myuname))
	{
		storage_set_registry(config_get_value_as_string(CONFIG_ITEM_ID_DUMMY_HOSTNAME),"pp/uname/sysname",myuname.sysname?myuname.sysname:"(NULL)",tm);
		storage_set_registry(config_get_value_as_string(CONFIG_ITEM_ID_DUMMY_HOSTNAME),"pp/uname/nodename",myuname.nodename?myuname.nodename:"(NULL)",tm);
		storage_set_registry(config_get_value_as_string(CONFIG_ITEM_ID_DUMMY_HOSTNAME),"pp/uname/release",myuname.release?myuname.release:"(NULL)",tm);
		storage_set_registry(config_get_value_as_string(CONFIG_ITEM_ID_DUMMY_HOSTNAME),"pp/uname/version",myuname.version?myuname.version:"(NULL)",tm);
		storage_set_registry(config_get_value_as_string(CONFIG_ITEM_ID_DUMMY_HOSTNAME),"pp/uname/machine",myuname.machine?myuname.machine:"(NULL)",tm);
#if defined (OS_LINUX) || defined(OS_OPENBSD) || defined(OS_DARWIN)
		storage_set_registry(config_get_value_as_string(CONFIG_ITEM_ID_DUMMY_HOSTNAME),"pp/uname/machine",myuname.machine?myuname.machine:"(NULL)",tm);
#endif
	}
}

void log_dropped_line(char*line) {
	FILE*fd;
	fd = log_file_fopen(config_get_value_as_string(CONFIG_ITEM_ID_DROP_FILE), time(0), config_get_value_as_boolean(CONFIG_ITEM_ID_DROP_FILE_ROTATE));
	if(!fd) fd = stderr;
	fprintf(fd,"%s\n",line);
	if(fd != stderr) fclose(fd);
}

int parse_and_store_perf_data_line(GString*l, perfparse_statistics_t*stats, log_line_t *log_data, perf_fields_t *perf_fields) {
	if(stats) stats->nb_lines_parsed++;
	//		printf("log_reader: '%s'\n",l->str);
	if(!line_split(l->str,log_data)) {
		if (!config_get_value_as_boolean(CONFIG_ITEM_ID_NO_RAW_DATA))
			storage_store_line(log_data);
		if(!perf_get_first(log_data,perf_fields)) {
			do {
				if (!config_get_value_as_boolean(CONFIG_ITEM_ID_NO_BIN_DATA)) {
					storage_store_perf(log_data,perf_fields);
					if(stats) stats->nb_metrics_parsed++;
				}
			} while(!perf_get_next(log_data,perf_fields));
		}
		//			printf("\n");
	} else {
		log_dropped_line(l->str);
		if(stats) stats->nb_lines_dropped++;
	}
	return(0);
}
